@@ -21,13 +21,16 @@ private:
     HCRYPTHASH hhash;
     Crypt *parent;
     Key *pkey;
-    void init(Crypt *ctx, Key *key) throw(CSPException);
+    void init(Crypt *ctx, Key *key, int length) throw(CSPException);
     HMAC_INFO   HmacInfo;
+    int length;
 protected:
     //
 public:
-    Hash(Crypt *ctx, BYTE *STRING, DWORD LENGTH, Key *key=0) throw(CSPException);
-    Hash(Crypt *ctx, Key *key=0) throw(CSPException);
+    Hash(Crypt *ctx, BYTE *STRING, DWORD LENGTH, Key *key, int length=0) throw(CSPException);
+    Hash(Crypt *ctx, BYTE *STRING, DWORD LENGTH, int length=0) throw(CSPException);
+    Hash(Crypt *ctx, Key *key, int length=0) throw(CSPException);
+    Hash(Crypt *ctx, int length=0) throw(CSPException);
 
     virtual ~Hash() throw(CSPException);
 
@@ -35,6 +38,7 @@ public:
     void update(BYTE *STRING, DWORD LENGTH) throw(CSPException);
     void sign(BYTE **s, DWORD *slen, DWORD dwKeyspec=AT_KEYEXCHANGE) throw(CSPException);
     bool verify(Cert *cert, BYTE *STRING, DWORD LENGTH) throw(CSPException);
+    bool verify(Key *key, BYTE *STRING, DWORD LENGTH) throw (CSPException);
     Key *derive_key() throw(CSPException);
 
     friend class Crypt;
