@@ -2,15 +2,28 @@
 #include "context.hpp"
 #include "cert.hpp"
 #include "key.hpp"
+#include <WinCryptEx.h>
 
-void Hash::init(Crypt *ctx, Key *key) throw(CSPException)
+ALG_ID hashCAlg(int length) {
+    if (2001 == length) {
+        return CALG_GR3411;
+    }
+    if (512 == length) {
+        return CALG_GR3411_2012_512;
+    }
+    return CALG_GR3411_2012_256;
+}
+
+void Hash::init(Crypt *ctx, Key *key, int l) throw(CSPException)
 {
+    HCRYPTKEY  hKey = key? key->hkey: 0;
     parent = 0;
     pkey = 0;
+    length = l;
     if(!CryptCreateHash(
         ctx->hprov,
-        (key? CALG_GR3411_HMAC : CALG_GR3411), 
-        (key? key->hkey : 0), 
+        hashCAlg(length), 
+        hKey, 
         0, 
         &hhash)) 
     {
@@ -52,17 +65,29 @@ void Hash::init(Crypt *ctx, Key *key) throw(CSPException)
  * * key -- `csp.Key`, если задан, хэш будет создан как HMAC
  *
  */
-Hash::Hash(Crypt *ctx, BYTE *STRING, DWORD LENGTH, Key *key) throw(CSPException)
+Hash::Hash(Crypt *ctx, BYTE *STRING, DWORD LENGTH, Key *key, int length) throw(CSPException)
 {
-    init(ctx, key);
+    init(ctx, key, length);
     this->update(STRING, LENGTH);
 }
 
-Hash::Hash(Crypt *ctx, Key *key) throw(CSPException)
+Hash::Hash(Crypt *ctx, BYTE *STRING, DWORD LENGTH, int length) throw(CSPException)
 {
-    init(ctx, key);
+    init(ctx, 0, length);
+    this->update(STRING, LENGTH);
 }
 
+Hash::Hash(Crypt *ctx, Key *key, int length) throw(CSPException)
+{
+    init(ctx, key, length);
+}
+
+Hash::Hash(Crypt *ctx, int length) throw(CSPException)
+{
+    init(ctx, 0, length);
+}
+
+
 Hash::~Hash() throw(CSPException)
 {
     if (hhash && !CryptDestroyHash(hhash)) {
@@ -146,7 +171,7 @@ void Hash::sign(BYTE **s, DWORD *slen, DWORD dwKeyspec) throw(CSPException)
  */
 bool Hash::verify(Cert *cert, BYTE *STRING, DWORD LENGTH) throw (CSPException)
 {
-    HCRYPTKEY hPubKey = NULL;
+    HCRYPTKEY hPubKey = 0;
     // Get the public key from the certificate
     if (!CryptImportPublicKeyInfo(
         parent->hprov, 
@@ -175,6 +200,29 @@ bool Hash::verify(Cert *cert, BYTE *STRING, DWORD LENGTH) throw (CSPException)
     return true;
 }
 
+bool Hash::verify(Key *key, BYTE *STRING, DWORD LENGTH) throw (CSPException)
+{
+    HCRYPTKEY hPubKey = 0;
+
+    if (!CryptVerifySignature(
+            hhash, 
+            STRING, 
+            LENGTH, 
+            key->hkey,
+            NULL, 
+            0
+        ))
+    {
+        DWORD err = GetLastError();
+        if (err == NTE_BAD_SIGNATURE) {
+            return false;
+        }
+        throw CSPException("Hash.verify(): error verifying hash", err);
+    }
+    return true;
+}
+
+
 Key *Hash::derive_key() throw(CSPException) 
 {
     HCRYPTKEY hkey;
